#include <stdio.h>
#include <stdlib.h>

#include "rbtree.h"

enum rbtree_color { RED, BLACK };

struct rbtree_node
   {
   enum rbtree_color color;
   struct rbtree_node * parent;
   struct rbtree_node * left;
   struct rbtree_node * right;
   void * data;
   };

struct rbtree
   {
   struct rbtree_node nil_node;
   struct rbtree_node * nil;
   struct rbtree_node * root;
   size_t size;
   bool (*le)(void * a, void * b);
   bool (*eq)(void * a, void * b);
   };

struct rbtree_iterator
   {
   struct rbtree * rbtree;
   struct rbtree_node * curr;
   };

// Private functions.
void rbtree_data_walk_inorder(struct rbtree * rbtree, struct rbtree_node * node, void (*op)(void * data));
void rbtree_data_walk_preorder(struct rbtree * rbtree, struct rbtree_node * node, void (*op)(void * data));
void rbtree_data_walk_postorder(struct rbtree * rbtree, struct rbtree_node * node, void (*op)(void * data));
void rbtree_node_walk_postorder(struct rbtree * rbtree, struct rbtree_node * node, void (*op)(struct rbtree_node * n));
void * rbtree_data_search(struct rbtree * rbtree, struct rbtree_node * node, void * data);
void * rbtree_node_minimum(struct rbtree * rbtree, struct rbtree_node * node);
void * rbtree_node_maximum(struct rbtree * rbtree, struct rbtree_node * node);
void rbtree_node_free(struct rbtree_node * node);
struct rbtree_node * rbtree_node_search(struct rbtree * rbtree, struct rbtree_node * node, void * data);
struct rbtree_node * rbtree_transplant(struct rbtree * rbtree, struct rbtree_node * a, struct rbtree_node * b);
void rbtree_rotate_left(struct rbtree * rbtree, struct rbtree_node * node);
void rbtree_rotate_right(struct rbtree * rbtree, struct rbtree_node * node);
void rbtree_insert_fixup(struct rbtree * rbtree, struct rbtree_node * node);
void rbtree_remove_fixup(struct rbtree * rbtree, struct rbtree_node * node);

////////////////////////////////////////////////////////////////////////////////
struct rbtree *
rbtree_new(bool (*le)(void * a, void * b),
	bool (*eq)(void * a, void * b))
   {
   if ( (NULL == le) || (NULL == eq) )
      return NULL;

   struct rbtree * rbtree = malloc(sizeof(struct rbtree));
   if (NULL == rbtree)
      return NULL;

   rbtree->nil_node = (struct rbtree_node) { BLACK, &rbtree->nil_node, &rbtree->nil_node, &rbtree->nil_node, NULL };
   rbtree->nil = &(rbtree->nil_node);
   rbtree->root = rbtree->nil;
   rbtree->size = 0;
   rbtree->le = le;
   rbtree->eq = eq;
   return rbtree;
   }


////////////////////////////////////////////////////////////////////////////////
void
rbtree_delete(struct rbtree * rbtree)
   {
   if (NULL == rbtree) return;
   if (0 < rbtree->size)
      {
      // Walk the tree and delete the nodes.
      // This may cause memory leaks, so hopefully the user has already
      // emptied the tree.
      rbtree_node_walk_postorder(rbtree, rbtree->root, rbtree_node_free);
      }
   free(rbtree);
   }

////////////////////////////////////////////////////////////////////////////////
void
rbtree_node_free(struct rbtree_node * node)
   {
   free(node);
   }

////////////////////////////////////////////////////////////////////////////////
void *
rbtree_insert(struct rbtree * rbtree, void * data)
   {
   if ( (NULL == rbtree) || (NULL == data) )
      return NULL;

   struct rbtree_node * new = malloc(sizeof(struct rbtree_node));
   if (NULL == new)
      return NULL;
   new->left = new->right = new->parent = rbtree->nil;
   new->color = RED;
   new->data = data;

   if (rbtree->nil == rbtree->root)
      {
      rbtree->root = new;
      new->color = BLACK;
      new->parent = rbtree->nil;
      rbtree->size = 1;
      return new->data;
      }

   struct rbtree_node * p = rbtree->nil;
   struct rbtree_node * c = rbtree->root;

   while (rbtree->nil != c)
      {
      p = c;
      if (rbtree->le(new->data, c->data))
	 c = c->left;
      else
	 c = c->right;
      }
   new->parent = p;
   if (rbtree->le(new->data, p->data))
      p->left = new;
   else
      p->right = new;

   rbtree_insert_fixup(rbtree, new);

   rbtree->size++;
   return new->data;
   }


////////////////////////////////////////////////////////////////////////////////
void
rbtree_insert_fixup(struct rbtree * rbtree, struct rbtree_node * node)
   {
   if ( (NULL == rbtree) || (NULL == node) )
      return;

   while (RED == node->parent->color)
      {
      if (node->parent == node->parent->parent->left)
	 {
	 struct rbtree_node * y = node->parent->parent->right;
	 if (RED == y->color)
	    {
	    node->parent->color = BLACK;
	    y->color = BLACK;
	    node->parent->parent->color = RED;
	    node = node->parent->parent;
	    }
	 else 
	    {
	    if (node == node->parent->right)
	       {
	       node = node->parent;
	       rbtree_rotate_left(rbtree, node);
	       }
	    node->parent->color = BLACK;
	    node->parent->parent->color = RED;
	    rbtree_rotate_right(rbtree, node->parent->parent);
	    }
	 }
      else
	 {
	 struct rbtree_node * y = node->parent->parent->left;
	 if (RED == y->color)
	    {
	    node->parent->color = BLACK;
	    y->color = BLACK;
	    node->parent->parent->color = RED;
	    node = node->parent->parent;
	    }
	 else 
	    {
	    if (node == node->parent->left)
	       {
	       node = node->parent;
	       rbtree_rotate_right(rbtree, node);
	       }
	    node->parent->color = BLACK;
	    node->parent->parent->color = RED;
	    rbtree_rotate_left(rbtree, node->parent->parent);
	    }
	 }
      }
   rbtree->root->color = BLACK;
   }

////////////////////////////////////////////////////////////////////////////////
void
rbtree_walk_inorder(struct rbtree * rbtree, void (*op)(void * data))
   {
   if ( (NULL == rbtree) || (rbtree->nil == rbtree->root) || (NULL == op) )
      return;

   rbtree_data_walk_inorder(rbtree, rbtree->root, op);
   }

////////////////////////////////////////////////////////////////////////////////
void
rbtree_walk_postorder(struct rbtree * rbtree, void (*op)(void * data))
   {
   if ( (NULL == rbtree) || (rbtree->nil == rbtree->root) || (NULL == op) )
      return;

   rbtree_data_walk_postorder(rbtree, rbtree->root, op);
   }

////////////////////////////////////////////////////////////////////////////////
void
rbtree_walk_preorder(struct rbtree * rbtree, void (*op)(void * data))
   {
   if ( (NULL == rbtree) || (rbtree->nil == rbtree->root) || (NULL == op) )
      return;

   rbtree_data_walk_preorder(rbtree, rbtree->root, op);
   }

////////////////////////////////////////////////////////////////////////////////
void
rbtree_data_walk_inorder(struct rbtree * rbtree, struct rbtree_node * node, void (*op)(void * data))
   {
   if (rbtree->nil != node)
      {
      rbtree_data_walk_inorder(rbtree, node->left, op);
      op(node->data);
      rbtree_data_walk_inorder(rbtree, node->right, op);
      }
   }

////////////////////////////////////////////////////////////////////////////////
void
rbtree_data_walk_postorder(struct rbtree * rbtree, struct rbtree_node * node, void (*op)(void * data))
   {
   if (rbtree->nil != node)
      {
      rbtree_data_walk_postorder(rbtree, node->left, op);
      rbtree_data_walk_postorder(rbtree, node->right, op);
      op(node->data);
      }
   }

////////////////////////////////////////////////////////////////////////////////
void
rbtree_data_walk_preorder(struct rbtree * rbtree, struct rbtree_node * node, void (*op)(void * data))
   {
   if (rbtree->nil != node)
      {
      op(node->data);
      rbtree_data_walk_preorder(rbtree, node->left, op);
      rbtree_data_walk_preorder(rbtree, node->right, op);
      }
   }

////////////////////////////////////////////////////////////////////////////////
void
rbtree_node_walk_postorder(struct rbtree * rbtree, struct rbtree_node * node, void (*op)(struct rbtree_node * n))
   {
   if (rbtree->nil != node)
      {
      rbtree_node_walk_postorder(rbtree, node->left, op);
      rbtree_node_walk_postorder(rbtree, node->right, op);
      op(node);
      }
   }

////////////////////////////////////////////////////////////////////////////////
void *
rbtree_search(struct rbtree * rbtree, void * data)
   {
   if ( (NULL == rbtree) || (rbtree->nil == rbtree->root) || (NULL == data) )
      return NULL;

   return rbtree_data_search(rbtree, rbtree->root, data);
   }

////////////////////////////////////////////////////////////////////////////////
// Recursive
void *
rbtree_data_search(struct rbtree * rbtree, struct rbtree_node * node, void * data)
   {
   if (rbtree->nil == node)
      return NULL;

   if (rbtree->eq(data, node->data))
      return node->data;

   if (rbtree->le(data, node->data))
      return rbtree_data_search(rbtree, node->left, data);
   else
      return rbtree_data_search(rbtree, node->right, data);
   }

////////////////////////////////////////////////////////////////////////////////
// Iterative
void *
rbtree_search_iterative(struct rbtree * rbtree, void * data)
   {
   if ( (NULL == rbtree) || (rbtree->nil == rbtree->root) || (NULL == data) )
      return NULL;

   struct rbtree_node * node = rbtree->root;
   while (1)
      {
      if (rbtree->nil == node)
	 return NULL;
      
      if (rbtree->eq(data, node->data))
	 return node->data;
      
      if (rbtree->le(data, node->data))
	 node = node->left;
      else
	 node = node->right;
      }
   }

////////////////////////////////////////////////////////////////////////////////
void *
rbtree_minimum(struct rbtree * rbtree)
   {
   if (NULL == rbtree)
      return NULL;

   return rbtree_node_minimum(rbtree, rbtree->root);
   }

////////////////////////////////////////////////////////////////////////////////
void *
rbtree_node_minimum(struct rbtree * rbtree, struct rbtree_node * node)
   {
   while ( (rbtree->nil != node) && (rbtree->nil != node->left) )
      node = node->left;
   if (rbtree->nil == node)
      return NULL;
   return node->data;
   }

////////////////////////////////////////////////////////////////////////////////
void *
rbtree_maximum(struct rbtree * rbtree)
   {
   if (NULL == rbtree)
      return NULL;

   return rbtree_node_maximum(rbtree, rbtree->root);
   }

////////////////////////////////////////////////////////////////////////////////
void *
rbtree_node_maximum(struct rbtree * rbtree, struct rbtree_node * node)
   {
   while ( (rbtree->nil != node) && (rbtree->nil != node->right) )
      node = node->right;
   if (rbtree->nil == node)
      return NULL;
   return node->data;
   }

////////////////////////////////////////////////////////////////////////////////
struct rbtree_iterator *
rbtree_begin(struct rbtree * rbtree)
   {
   if ( (NULL == rbtree) || (rbtree->nil == rbtree->root) )
      return NULL;

   struct rbtree_node * min = rbtree->root;
   while (min->left)
      min = min->left;

   struct rbtree_iterator * it = malloc(sizeof(struct rbtree_iterator));
   if (NULL == it)
      return NULL;

   it->rbtree = rbtree;
   it->curr = min;
   return it;
   }

////////////////////////////////////////////////////////////////////////////////
struct rbtree_iterator *
rbtree_end(struct rbtree * rbtree)
   {
   if ( (NULL == rbtree) || (rbtree->nil == rbtree->root) )
      return NULL;

   struct rbtree_node * max = rbtree->root;
   while (max->right)
      max = max->right;

   struct rbtree_iterator * it = malloc(sizeof(struct rbtree_iterator));
   if (NULL == it)
      return NULL;

   it->rbtree = rbtree;
   it->curr = max;
   return it;
   }

////////////////////////////////////////////////////////////////////////////////
struct rbtree_iterator *
rbtree_next(struct rbtree_iterator * it)
   {
   if ( (NULL == it) || (NULL == it->rbtree) || (it->rbtree->nil == it->curr) )
      return NULL;

   if (it->rbtree->nil == it->curr->right)
      {
      struct rbtree_node * p = it->curr;
      it->curr = it->curr->parent;
      while ( (it->rbtree->nil != it->curr) && (p == it->curr->right) )
	 {
	 p = it->curr;
	 it->curr = it->curr->parent;
	 }

      if (it->curr == it->rbtree->nil)
	 return NULL;
      }
   else
      {
      struct rbtree_node * min = it->curr->right;
      while (min->left)
	 min = min->left;
      it->curr = min;
      }
   return it;
   }

////////////////////////////////////////////////////////////////////////////////
struct rbtree_iterator *
rbtree_prev(struct rbtree_iterator * it)
   {
   if ( (NULL == it) || (NULL == it->rbtree) || (it->rbtree->nil == it->curr) )
      return NULL;

   if (it->rbtree->nil == it->curr->left)
      {
      struct rbtree_node * p = it->curr;
      it->curr = it->curr->parent;
      while ( (it->rbtree->nil != it->curr) && (p == it->curr->left) )
	 {
	 p = it->curr;
	 it->curr = it->curr->parent;
	 }

      if (it->curr == it->rbtree->nil)
	 return NULL;
      }
   else
      {
      struct rbtree_node * max = it->curr->left;
      while (max->right)
	 max = max->right;
      it->curr = max;
      }
   return it;
   }

////////////////////////////////////////////////////////////////////////////////
void *
rbtree_value(struct rbtree_iterator * it)
   {
   if ( (NULL == it) || (NULL == it->rbtree) || (it->rbtree->nil == it->curr) )
      return NULL;

   return it->curr->data;
   }

////////////////////////////////////////////////////////////////////////////////
void *
rbtree_find(struct rbtree_iterator * it, void * data)
   {
   if ( (NULL == it) || (NULL == it->rbtree) || (it->rbtree->nil == it->rbtree->root) || (NULL == data) )
      return NULL;

   return rbtree_node_search(it->rbtree, it->rbtree->root, data);
   }

////////////////////////////////////////////////////////////////////////////////
// Recursive
struct rbtree_node *
rbtree_node_search(struct rbtree * rbtree, struct rbtree_node * node, void * data)
   {
   if (rbtree->nil == node)
      return NULL;

   if (rbtree->eq(data, node->data))
      return node;

   if (rbtree->le(data, node->data))
      return rbtree_node_search(rbtree, node->left, data);
   else
      return rbtree_node_search(rbtree, node->right, data);
   }

////////////////////////////////////////////////////////////////////////////////
struct rbtree_node *
rbtree_transplant(struct rbtree * rbtree, struct rbtree_node * a, struct rbtree_node * b)
   {
   if ( (NULL == a) || (NULL == b) )
      return NULL;

   if (rbtree->nil == a->parent)
      rbtree->root = b;
   else if (a == a->parent->left)
      a->parent->left = b;
   else
      a->parent->right = b;

   b->parent = a->parent;

   return a;
   }

////////////////////////////////////////////////////////////////////////////////
void *
rbtree_remove(struct rbtree_iterator * it)
   {
   if ( (NULL == it) || (NULL == it->curr) )
      return NULL;

   // Get predecessor and save it
   struct rbtree_iterator pred;
   pred.curr = it->curr;
   pred.rbtree = it->rbtree;
   rbtree_prev(&pred);

   struct rbtree_node * x;
   struct rbtree_node * y = it->curr;
   enum rbtree_color original_color = y->color;

   if (it->rbtree->nil == it->curr->left)
      {
      x = it->curr->right;
      rbtree_transplant(it->rbtree, it->curr, it->curr->right);
      }
   else if (it->rbtree->nil == it->curr->right)
      {
      x = it->curr->left;
      rbtree_transplant(it->rbtree, it->curr, it->curr->left);
      }
   else
      {
      y = rbtree_node_minimum(it->rbtree, it->curr->right);
      original_color = y->color;
      x = y->right;
      if (it->curr == y->parent)
	 x->parent = y;
      else
	 {
	 rbtree_transplant(it->rbtree, y, y->right);
	 y->right = it->curr->right;
	 y->right->parent = y;
	 }
      rbtree_transplant(it->rbtree, it->curr, y);
      y->left = it->curr->left;
      y->left->parent = y;
      y->color = it->curr->color;
      }

   if (BLACK == original_color)
      rbtree_remove_fixup(it->rbtree, x);

   void * data = it->curr->data;
   free(it->curr);

   // Update it to point to pred's successor
   rbtree_next(&pred);
   it->curr = pred.curr;

   return data;
   }

////////////////////////////////////////////////////////////////////////////////
void
rbtree_remove_fixup(struct rbtree * rbtree, struct rbtree_node * node)
   {
   while ( (rbtree->root != node) && (BLACK == node->color) )
      {
      if (node == node->parent->left)
	 {
	 struct rbtree_node * w = node->parent->right;
	 if (RED == w->color)
	    {
	    w->color = BLACK;
	    node->parent->color = RED;
	    rbtree_rotate_left(rbtree, node->parent);
	    w = node->parent->right;
	    }
	 if ( (BLACK == w->left->color) && (BLACK == w->right->color) )
	    {
	    w->left->color = RED;
	    node = node->parent;
	    }
	 else
	    {
	    if (BLACK == w->right->color)
	       {
	       w->left->color = BLACK;
	       w->color = RED;
	       rbtree_rotate_right(rbtree, w);
	       w = node->parent->right;
	       }
	    w->color = node->parent->color;
	    node->parent->color = BLACK;
	    w->right->color = BLACK;
	    rbtree_rotate_left(rbtree, node->parent);
	    node = rbtree->root;
	    }
	 }
      else
	 {
	 struct rbtree_node * w = node->parent->left;
	 if (RED == w->color)
	    {
	    w->color = BLACK;
	    node->parent->color = RED;
	    rbtree_rotate_right(rbtree, node->parent);
	    w = node->parent->left;
	    }
	 if ( (BLACK == w->right->color) && (BLACK == w->left->color) )
	    {
	    w->right->color = RED;
	    node = node->parent;
	    }
	 else
	    {
	    if (BLACK == w->left->color)
	       {
	       w->right->color = BLACK;
	       w->color = RED;
	       rbtree_rotate_left(rbtree, w);
	       w = node->parent->left;
	       }
	    w->color = node->parent->color;
	    node->parent->color = BLACK;
	    w->left->color = BLACK;
	    rbtree_rotate_right(rbtree, node->parent);
	    node = rbtree->root;
	    }
	 }
      }
   node->color = BLACK;
   }

////////////////////////////////////////////////////////////////////////////////
void
rbtree_rotate_left(struct rbtree * rbtree, struct rbtree_node * node)
   {
   if ( (NULL == rbtree) || (rbtree->nil == node) || (rbtree->nil == node->right) )
      return;

   struct rbtree_node * y = node->right;

   node->right = y->left;
   if (rbtree->nil != y->left)
      y->left->parent = node;

   y->parent = node->parent;
   if (rbtree->nil == node->parent)
      rbtree->root = y;
   else if (node == node->parent->left)
      node->parent->left = y;
   else
      node->parent->right = y;

   y->left = node;
   node->parent = y;
   }

////////////////////////////////////////////////////////////////////////////////
void
rbtree_rotate_right(struct rbtree * rbtree, struct rbtree_node * node)
   {
   if ( (NULL == rbtree) || (rbtree->nil == node) || (rbtree->nil == node->left) )
      return;

   struct rbtree_node * y = node->left;

   node->left = y->right;
   if (rbtree->nil != y->right)
      y->right->parent = node;

   y->parent = node->parent;
   if (rbtree->nil == node->parent)
      rbtree->root = y;
   else if (node == node->parent->right)
      node->parent->right = y;
   else
      node->parent->left = y;

   y->right = node;
   node->parent = y;
   }

